"""Exceptions.

"""


class CompileError(Exception):
    """General ASN.1 compile error.

    """


class EncodeError(Exception):
    """General ASN.1 encode error.

    """


class DecodeError(Exception):
    """General ASN.1 decode error.

    """
